package kd.hrmp.hbjm.business.cascadedata;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.utils.ObjectUtils;
import kd.bos.exception.KDException;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;

import java.util.Map;
import java.util.Optional;

/**
 * @author rd_ningjie
 * @description:级联数据对比服务类
 * @date 2022/6/6 9:53
 */
public class CascadeDataValideService implements AutoCloseable {

    private static final Log LOG = LogFactory.getLog(CascadeDataValideService.class);

    //实体
    private CompareEntity entity;

    public CascadeDataValideService(CompareEntity entity) {
        this.entity = entity;
    }

    /**
     * @description: 初始化数据
     * @return: void
     * @author rd_ningjie
     * @date: 2022/6/6 11:37
     */
    public CascadeDataValideService initData() {
        Optional.ofNullable(this.entity).ifPresent(entity -> {
            this.entity.initData();
        });
        return this;
    }

    /**
     * @param mainCompareValue
     * @param subCompareValue
     * @description: 采用字符串比对，long类型等请采用String.valueOf()转换一下
     * @return: boolean
     * @author rd_ningjie
     * @date: 2022/6/6 14:58
     */
    public boolean compare(String mainCompareValue, String subCompareValue) {

        //如果有实体为空，则直接抛出异常
        if (entity == null) {
            throw new RuntimeException("params can not be null");
        }

        //如果对比值不存在，则直接返回不在集合中的结果
        if (ObjectUtils.isEmpty(mainCompareValue)) {
            return false;
        }
        //如果下级没有填，那就直接返回true，由其他业务进行必填性校验
        if (ObjectUtils.isEmpty(subCompareValue)) {
            return true;
        }

        this.entity.setValue(mainCompareValue);

        this.entity.setCompareValue(subCompareValue);

        //对比数据
        return this.valideDatas();

    }

    /**
     * @description: 校验数据有效性
     * @return: boolean
     * @author rd_ningjie
     * @date: 2022/6/6 14:09
     */
    protected boolean valideDatas() {

        Map<String, DynamicObject> mainMap = entity.getDataContainer();
        //如果子集合数据源有为空，则直接返回不在集合中的结果
        if (mainMap == null || mainMap.size() <= 0) {
            return false;
        }
        //通过联合主键去map中查询
        if (mainMap.containsKey(this.entity.getValue() + this.entity.CON_SYMBOL + this.entity.getCompareValue()))
            return true;

        return false;
    }

    @Override
    public void close() throws KDException {
        LOG.info("　CascadeDataValideService　　close　start　释放资源。。。。。。。。。。。。。。。。。。");
        //清空数据源，释放内存
        Optional.ofNullable(entity).ifPresent(entity -> {
            entity.setDataContainer(null);
            this.entity = null;
        });
        LOG.info("　CascadeDataValideService　　close　end　释放资源。。。。。。。。。。。。。。。。。。");
    }

    public CompareEntity getEntity() {
        return entity;
    }
}
