package kd.hrmp.hbjm.business.domain.application.impl.job;

import kd.bos.context.RequestContext;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.ILocaleString;
import kd.bos.dataentity.metadata.IDataEntityProperty;
import kd.bos.dataentity.utils.ObjectUtils;
import kd.bos.entity.property.MuliLangTextProp;
import kd.bos.entity.property.TextProp;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.bos.orm.query.QFilter;
import kd.hr.hbp.business.servicehelper.HRBaseServiceHelper;
import kd.hr.hbp.common.constants.HRBaseConstants;
import kd.hr.hbp.common.constants.newhismodel.HisFieldNameConstants;
import kd.hrmp.hbjm.business.domain.application.job.IJobApplication;
import kd.hrmp.hbjm.business.domain.repository.HBJMEventRepository;
import kd.hrmp.hbjm.business.domain.repository.JobRepository;
import kd.hrmp.hbjm.business.domain.service.IJobService;
import kd.hrmp.hbjm.business.domain.service.impl.JobServiceImpl;
import kd.hrmp.hbjm.business.model.JobChangeInfoModel;
import kd.hrmp.hbjm.business.utils.JobDynamicObjectUtils;
import kd.hrmp.hbjm.business.utils.model.JobDisableModel;
import kd.hrmp.hbjm.common.constants.HBJMConstants;
import kd.hrmp.hbjm.common.constants.JobConstans;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author rd_ningjie
 * @description:职位相关聚合服务实现类
 * @date 2022/6/16 10:36
 */
public class JobApplicationImpl implements IJobApplication {

    private static final Log LOGGER = LogFactory.getLog(JobApplicationImpl.class);

    //线程变量，存储当前线程已经解析过的实体，key值为实体编码 value值为非数据库字段map
    protected static ThreadLocal<Map<String, Map<String, IDataEntityProperty>>> dynamicObjectContainer =
            ThreadLocal.withInitial(() -> new HashMap<>());


    /**
     * @param sourceData
     * @param viewData
     * @description: 处理职位导入职级职等信息被置空的逻辑
     * @return: DynamicObject
     * @author rd_ningjie
     * @date: 2022/6/16 10:49
     */
    @Override
    public DynamicObject dealGradeOrLevelImportToNull(Map<String, Object> sourceData, DynamicObject viewData) {

        if (ObjectUtils.isEmpty(sourceData)) {
            return viewData;
        }

        String entityCode = viewData.getDataEntityType().getName();
        Map<String, IDataEntityProperty> props = Optional.ofNullable(dynamicObjectContainer.get().get(entityCode)).orElseGet(() -> {
            //获取非数据库字段信息
            Map<String, IDataEntityProperty> infos = JobDynamicObjectUtils.getNoDbCol(viewData);
            dynamicObjectContainer.get().put(entityCode, infos);
            return infos;
        });
        //如果sourceData数据中存在此字段（导入中带有此字段），而且是置为null，就将此字段设置为特殊标识，便于后台进行校验等处理（由于不绑定数据库字段，平台不会做置空的处理）
        if (ObjectUtils.isEmpty(props)) {
            return viewData;
        }


        Iterator<Map.Entry<String, IDataEntityProperty>> iterator = props.entrySet().iterator();
        while (iterator.hasNext()) {
            Map.Entry<String, IDataEntityProperty> entry = iterator.next();
            String key = entry.getKey();
            //如果sourcedata中被设置为空，则置空viewDatad的数据
            Object data = sourceData.get(key);
            if (sourceData.containsKey(key)) {
                //分别处理多语言字段 文本字段类型 （分录、基础资料 、多选基础资料一定会绑定数据库字段，此处不做处理）
                changeViewData(key, entry.getValue(), data, viewData);
            }
        }
        return viewData;
    }

    /**
     * @param sourceData
     * @param viewData
     * @description: 处理职位导入变动原因置空的逻辑
     * @return: DynamicObject
     * @author rd_ningjie
     * @date: 2022/6/16 10:49
     */
    @Override
    public DynamicObject dealChangeReasonImportToNull(Map<String, Object> sourceData, DynamicObject viewData) {
        if (ObjectUtils.isEmpty(sourceData)) {
            return viewData;
        }

        if (!sourceData.containsKey(JobConstans.JOB_CHANGE_DESC)) {
            viewData.set(JobConstans.JOB_CHANGE_DESC, null);
        }
        if (!sourceData.containsKey(JobConstans.JOB_CHANGEDESC2_COL)) {
            viewData.set(JobConstans.JOB_CHANGEDESC2_COL, null);
        }
        return viewData;
    }

    /**
     * @param boids
     * @description: 根据boid查询所有的历史数据以及对应的变动原因变动类型等事务数据
     * @return: DynamicObject
     * @author rd_ningjie
     * @date: 2022/6/16 10:49
     */
    @Override
    public Map<Long, List<JobChangeInfoModel>> getAllChangeInfoByBoids(Set<Long> boids) {

        if (ObjectUtils.isEmpty(boids)) {
            return null;
        }

        //职位变更记录
        DynamicObject[] jobhis = JobRepository.getInstance().findJobsChangeHis(boids);
        if (ObjectUtils.isEmpty(jobhis)) {
            return null;
        }

        //先查询所有的操作明细数据，再查询所有的事务明细数据，然后查询职位主数据，进行组装
        //操作明细
        Map<Long, DynamicObject> opsMap = new HashMap<>();
        Map<Long, DynamicObject> eventsMap = new HashMap<>();

        //获取变动操作明细
        DynamicObject[] ops = HBJMEventRepository.getInstance().selectOpDetailByBoIds(boids);
        DynamicObject[] eventDetails = null;
        if (!ObjectUtils.isEmpty(ops)) {
            //已变动后职位数据版本为key构建map
            opsMap = Arrays.stream(ops).collect(Collectors.toMap(op -> op.getLong("afterjob"), vv -> vv, (v1, v2) -> v2));

            //查询变动事务数据
            Set<Long> eventIds = Arrays.stream(ops).map(op -> op.getLong("event")).collect(Collectors.toSet());
            //事务变动详情
            eventDetails = HBJMEventRepository.getInstance().selectEventDetails(eventIds);

            if (!ObjectUtils.isEmpty(eventDetails)) {
                //以变动事务id为key构建map
                eventsMap =
                        Arrays.stream(eventDetails).collect(Collectors.toMap(op -> op.getLong("event"), vv -> vv, (v1, v2) -> v2));
            }

        }

        //构建返回实体集合
        List<JobChangeInfoModel> resultTmp = new ArrayList<>(jobhis.length);
        for (DynamicObject his : jobhis) {
            JobChangeInfoModel model = new JobChangeInfoModel();
            model.setBoid(his.getLong(HisFieldNameConstants.BOID));
            model.setDataId(his.getLong(HRBaseConstants.ID));
            model.setJobHis(his);
            if (!ObjectUtils.isEmpty(opsMap)) {
                Long id = his.getLong("id");
                DynamicObject opDetail = opsMap.get(id);
                if (!ObjectUtils.isEmpty(opDetail)) {
                    model.setOpDetail(opDetail);
                    Long eventId = opDetail.getLong("event");
                    if (!ObjectUtils.isEmpty(eventsMap)) {
                        DynamicObject aaa = eventsMap.get(eventId);
                        if (aaa != null) {
                            model.setEventDetail(aaa);
                        }
                    }
                }
            }
            resultTmp.add(model);
        }

        //boid为key值的职位历史数据
        Map<Long, List<JobChangeInfoModel>> result
                = resultTmp.stream().collect(Collectors.groupingBy(JobChangeInfoModel::getBoid));

        return result;
    }

    private void changeViewData(String key, IDataEntityProperty prop, Object source, DynamicObject viewData) {

        if (prop instanceof MuliLangTextProp) {
            //如果sourcedata中被设置为空，则置空viewDatad的语言环境数据
            Map<String, Object> data = (Map) source;
            ILocaleString dataresult = viewData.getLocaleString("key");

            Iterator<Map.Entry<String, Object>> iterator = data.entrySet().iterator();
            while (iterator.hasNext()) {
                Map.Entry<String, Object> entry = iterator.next();
                String kk = entry.getKey();
                Object val = entry.getValue();
                //强制为空==null才做处理
                if (val == null) {
                    dataresult.setItem(kk, HBJMConstants.TO_NULL_COIN_STR);
                }
            }
            viewData.set(key, dataresult);
        } else if (prop instanceof TextProp) {
            if (source == null) {
                viewData.set(key, HBJMConstants.TO_NULL_COIN_STR);
            }
        }
    }

    /**
     * @param jobs
     * @description: 禁用职位
     * @return: void
     * @author rd_ningjie
     * @date: 2022/6/17 17:03
     */
    @Override
    public DynamicObject[] disableJob(DynamicObject[] jobs, JobDisableModel models) {
        if (!ObjectUtils.isEmpty(jobs) && !ObjectUtils.isEmpty(models)) {
            List<Long> idList = Arrays.stream(jobs).map(job -> job.getLong(HRBaseConstants.ID))
                    .collect(Collectors.toList());
            String entityNumber = jobs[0].getDataEntityType().getName();
            //重新查出数据，并改变它的变动信息值
            QFilter idFilter = new QFilter(HRBaseConstants.ID, QFilter.in, idList);
            HRBaseServiceHelper jobHepler = new HRBaseServiceHelper(entityNumber);
            jobs = jobHepler.loadDynamicObjectArray(new QFilter[]{idFilter});
            if (!ObjectUtils.isEmpty(jobs)) {
                for (DynamicObject job : jobs) {
                    job.set(JobConstans.JOB_CHANGE_DESC, models.getChangedesc());
                    job.set(JobConstans.JOB_CHANGEDESC2_COL, models.getChangedesc2());
                    job.set(JobConstans.DISABLEDATE_COL, models.getHisDisableDate());
                    job.set(HRBaseConstants.ENABLE, HRBaseConstants.DISABLED);
                    job.set(JobConstans.DISABLER, RequestContext.get().getCurrUserId());
                    job.set(HisFieldNameConstants.EFF_START_DATE, models.getHisDisableDate());
                }
            }
            IJobService jobService = new JobServiceImpl();
            return jobService.batchHisVersionChange(jobs);
        }
        return jobs;
    }

    /**
     * @param jobs
     * @description: 启用职位
     * @return: DynamicObject[]
     * @author rd_ningjie
     * @date: 2022/6/17 17:03
     */
    @Override
    public DynamicObject[] enableJob(DynamicObject[] jobs) {
        if (!ObjectUtils.isEmpty(jobs)) {
            List<Long> idList = Arrays.stream(jobs).map(job -> job.getLong(HRBaseConstants.ID))
                    .collect(Collectors.toList());
            String entityNumber = jobs[0].getDataEntityType().getName();
            //重新查出数据，并改变它的变动信息值
            QFilter idFilter = new QFilter(HRBaseConstants.ID, QFilter.in, idList);
            HRBaseServiceHelper jobHepler = new HRBaseServiceHelper(entityNumber);
            jobs = jobHepler.loadDynamicObjectArray(new QFilter[]{idFilter});
            if (!ObjectUtils.isEmpty(jobs)) {
                for (DynamicObject job : jobs) {
                    job.set(JobConstans.JOB_CHANGE_DESC, null);
                    job.set(JobConstans.JOB_CHANGEDESC2_COL, null);
                    job.set(HRBaseConstants.ENABLE, HRBaseConstants.ENABLED);
                }
            }
            IJobService jobService = new JobServiceImpl();
            return jobService.batchHisVersionChange(jobs);
        }
        return jobs;
    }
}
