package kd.hrmp.hbjm.business.domain.repository;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.hr.hbp.business.servicehelper.HRBaseServiceHelper;
import kd.hr.hbp.common.constants.HRBaseConstants;
import kd.hr.hbp.common.constants.HRHisBaseConstants;
import kd.hr.hbp.common.constants.newhismodel.HisFieldNameConstants;
import kd.hrmp.hbjm.business.utils.QFilterCreatorUtils;
import kd.hrmp.hbjm.common.constants.HBJMConstants;
import kd.hrmp.hbjm.common.constants.JobConstans;

import java.util.List;
import java.util.Set;

/**
 * @author rd_xiangquan_zheng
 * @since 2022/3/1 15:36
 * @description 职等方案下数据库操作Repository
 */
public class JobGradeScmRepository {
    private static class JobGradeScmInstance {
        private static JobGradeScmRepository INSTANCE = new JobGradeScmRepository();
    }

    public static JobGradeScmRepository getInstance() {
        return JobGradeScmRepository.JobGradeScmInstance.INSTANCE;
    }

    /**
     * 根据职等方案id集合查询职等集合
     * @param rowIds 职等方案rowIds
     * @return dyns 职等集合
     */
    public DynamicObject[] queryJobGradeScmByrowIds(List<Long> rowIds) {
        HRBaseServiceHelper hrBaseServiceHelper = new HRBaseServiceHelper("hbjm_jobgradescmhr");
        QFilter jobGradeFilter = new QFilter("id", QFilter.in, rowIds);
        return hrBaseServiceHelper.query("id, boid, entryentity.entryboid, entryentity.jobgrade_number, entryentity.jobgrade_name, entryentity.jobgrade_seq",
                new QFilter[] {jobGradeFilter});
    }

    /**
     * 根据职等方案ids查询职等及职等方案信息
     * @param gradescmids 职等方案ids
     * @return 职等
     */
    public DynamicObject[] getGradeScms(Set<Long> gradescmids) {
        HRBaseServiceHelper scmgradeserviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBGRADESCM);
        QFilter gradeidFilter = new QFilter(HRBaseConstants.ID, QFilter.in, gradescmids);
        return scmgradeserviceHelper.query("entryentity,entryentity.jobgrade_number,entryentity.jobgrade_name,entryentity.jobgrade_seq", new QFilter[]{gradeidFilter});
    }

    /**
     * 根据职等方案number查询职等及职等方案信息
     *
     * @param gradeScmNumberSet 职等方案number Set
     * @return 职等
     */
    public DynamicObject[] getGradeScmDynsByNumberSet(Set<String> gradeScmNumberSet) {
        HRBaseServiceHelper gradeScmServiceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBGRADESCM);
        QFilter gradeNumberFilter = new QFilter(HRBaseConstants.NUMBER, QFilter.in, gradeScmNumberSet);
        return gradeScmServiceHelper.query("id, number, boid, createorg", new QFilter[]{gradeNumberFilter, QFilterCreatorUtils.createCurrentVersionFilter()});
    }

    /**
     * 根据职等ids查询职等
     * @param gradeIds 职等ids
     * @return 职等
     */
    public DynamicObject[] getJobGradeById(Set<Long> gradeIds) {
        HRBaseServiceHelper scmgradeserviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBGRADE);
        QFilter gradeidFilter = new QFilter(HRBaseConstants.ID, QFilter.in, gradeIds);
        return scmgradeserviceHelper.loadDynamicObjectArray(new QFilter[]{gradeidFilter});
    }

    /**
     * 根据职位方案boid查询历史版本
     * @param boidList
     * @return
     */
    public DynamicObject[] getJobGradeScmByBoId(Long[] boidList) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBGRADESCM);
        QFilter boidQFilter = new QFilter(HisFieldNameConstants.BOID, QCP.in, boidList);
        QFilter iscurrentversionQfilter = new QFilter(JobConstans.IS_CURRENT_VERSION, QCP.equals, "0");
        QFilter datastatusQFilter = new QFilter(HRHisBaseConstants.FIELD_DATASTATUS, QCP.in,
                new String[]{HRHisBaseConstants.EFFECTSTATUS_SCHEDULEEFFECT, HRHisBaseConstants.EFFECTSTATUS_EFFECTING,
                        HRHisBaseConstants.EFFECTSTATUS_INVALID});
        QFilter initStatusFilter = new QFilter(HBJMConstants.INIT_STATUS, QFilter.equals, HBJMConstants.INIT_STATUS_COMPLETE);
        QFilter qFilter = boidQFilter.and(datastatusQFilter).and(iscurrentversionQfilter).and(initStatusFilter);
        return serviceHelper.loadDynamicObjectArray(new QFilter[]{qFilter});
    }

    /**
     * 根据id查询权限范围内的数据
     * @param idList
     * @param permFilter
     * @return
     */
    public DynamicObject[] getJobGradeScmById(Set<Long> idList,QFilter permFilter) {
        HRBaseServiceHelper gradeScmServiceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBGRADESCM);
        QFilter idFilter = new QFilter(HRBaseConstants.ID, QFilter.in, idList);
        return gradeScmServiceHelper.query("id, number, boid", new QFilter[]{idFilter,permFilter});
    }
}
